from __future__ import print_function
import random
import boto3
from botocore.config import Config
from datetime import datetime
from .app_mxnet_inference import get_dma


def get_purchases(table, uid):
    get_purchase_ind = "n/a"
    try:
        response = table.get_item(Key={'userId': uid})
        if 'Item' in response:
            results = response['Item']['purchases']
            ind = 0
        else:
            results = []
            ind = 1
            get_purchase_ind = "noHistory"
    except Exception as e:
        results = []
        ind = 2
        get_purchase_ind = str(e)

    return results, ind, get_purchase_ind


def draw_test_group(test_group_weights):
    group_names = test_group_weights.get("groups")
    weights = test_group_weights.get("weights")
    assert len(group_names) == len(weights), "incomparable length groups and weights"

    groups = [
        random.choices(population=group_name, weights=weight)[0]
        for group_name, weight in zip(group_names, weights)
    ]
    return "-".join(groups)


def draw_national_group(national_group_weights):
    group_names = national_group_weights.get("groups")
    weights = national_group_weights.get("weights")
    assert len(group_names) == len(weights), "incomparable length groups and weights"

    group = random.choices(population=group_names, weights=weights)[0]
    return group


def draw_control(test_control_weights):
    return random.choices(
        population=[False, True],
        weights=[test_control_weights["test"], test_control_weights["control"]],
        k=1
    )[0]


def get_user_profile(uid, create_new_control, test_control_weights, rest_tz):
    my_config = Config(region_name='us-east-1', connect_timeout=2, read_timeout=4, retries={'max_attempts': 2})
    dynamodb = boto3.resource('dynamodb', config=my_config)
    profile_table = dynamodb.Table("digitalGuestProfile")
    try:
        response = profile_table.get_item(Key={'userId': uid})
        # user exists
        if 'Item' in response:
            # get purchase history
            if 'purchases' in response['Item']:
                purchase_history = response['Item']['purchases']
                get_purchase_ind = 'success'
                ind = 0
            else:
                purchase_history = []
                get_purchase_ind = 'noHistory'
                ind = 1

            # get loyalty_id
            loyalty_id = response.get("Item", {}).get("loyaltyId")

            # get control
            dt_now = datetime.utcnow()
            old_expires = response['Item'].get('assignedOffers', {}).get("weeklyOffersExpireOn", "2000-01-01T00:00:00")
            dt_old_expires = datetime.fromisoformat(old_expires.split(".")[0])
            is_newly_assigned_weekly = (dt_old_expires - dt_now).total_seconds() > 3600 * 24 * 4  # 4 days
            if "is_control" in response["Item"].get("assignedOffers", {}) and (
                    not create_new_control or is_newly_assigned_weekly):
                is_control = response["Item"].get("assignedOffers", {}).get("is_control")
                if type(is_control) != bool:
                    is_control = draw_control(test_control_weights=test_control_weights)
            else:
                is_control = draw_control(test_control_weights=test_control_weights)

            # # get group
            # if 'group' in response['Item'].get("assignedOffers", {}) and (
            #         not CREATE_NEW_GROUP or is_newly_assigned_weekly):
            #     control_group = response['Item']['assignedOffers']['group']
            #     get_group_ind = 'success'
            # else:
            #     control_group = draw_test_group(test_group_weights=TEST_GROUP_WEIGHTS)
            #     get_group_ind = 'newlyCreated'

            # get location
            user_location = response['Item'].get('user_attributes', {}).get('location', {})
        # user not exists
        else:
            loyalty_id = None
            user_attributes = None
            purchase_history = []
            user_location = {}
            get_purchase_ind = 'noHistory'
            is_newly_assigned_weekly = False
            ind = 1
            # control_group = draw_test_group(test_group_weights=TEST_GROUP_WEIGHTS)
            is_control = draw_control(test_control_weights=test_control_weights)
            # get_group_ind = 'newlyCreated'
    except Exception as e:
        loyalty_id = None
        user_attributes = None
        purchase_history = []
        user_location = {}
        ind = 2
        get_purchase_ind = str(e)
        is_newly_assigned_weekly = False
        # control_group = draw_test_group(test_group_weights=TEST_GROUP_WEIGHTS)
        is_control = draw_control(test_control_weights=test_control_weights)
        # get_group_ind = str(e) + 'newlyCreated'

    store_history = [str(purchase.get("bkid", "0")) for purchase in purchase_history]
    user_tier, is_phoenix = get_dma(store_ids=store_history, user_location=user_location, dim_rest=rest_tz)

    return loyalty_id, purchase_history, is_newly_assigned_weekly, get_purchase_ind, user_tier, is_control, is_phoenix


# def get_purchases_and_offers(table, uid):
#     get_purchase_ind = "n/a"
#     try:
#         response = table.get_item(Key={'userId': uid})
#         if 'Item' in response:
#             results = response['Item']['purchases']
#             assigned_offers = response["Item"]["assignedOffers"]
#             ind = 0
#         else:
#             results = []
#             assigned_offers = {}
#             ind = 1
#             get_purchase_ind = "noHistory"
#     except Exception as e:
#         results = []
#         assigned_offers = {}
#         ind = 2
#         get_purchase_ind = str(e)
#
#     return results, ind, get_purchase_ind, assigned_offers


def update_profile_offers(uid, assigned_offers):
    my_config = Config(region_name='us-east-1', connect_timeout=2, read_timeout=4, retries={'max_attempts': 2})
    dynamodb = boto3.resource('dynamodb', config=my_config)
    table = dynamodb.Table("digitalGuestProfile")

    update_expression = "set #attrName = :attrValue"
    expr_attr_names = {"#attrName": "assignedOffers"}
    expr_attr_values = {":attrValue": assigned_offers}
    # if loyalty_id:
    #     update_expression += ", #attrNameA = :attrValue1"
    #     expr_attr_names.update({"#attrNameA": "loyaltyId"})
    #     expr_attr_values.update({":attrValue1": loyalty_id})
    # if user_attributes:
    #     update_expression += ", #attrNameB = :attrValue2"
    #     expr_attr_names.update({"#attrNameB": "user_attributes"})
    #     expr_attr_values.update({":attrValue2": user_attributes})

    try:
        update_response = table.update_item(
            Key={
                'userId': uid
            },
            UpdateExpression=update_expression,
            ExpressionAttributeNames=expr_attr_names,
            ExpressionAttributeValues=expr_attr_values,
            ReturnValues="UPDATED_NEW"
        )
        status_code = update_response['ResponseMetadata']['HTTPStatusCode']
    except Exception as e:
        status_code = str(e)
    return status_code


def get_test_group(default_offers, test_group_weights, is_control, user_tier, has_history):
    k1 = "control" if is_control else "personalized" if has_history else "default"
    k2 = "tier1" if user_tier == "1" else "tier2"
    price_tier = "1_Personalized" if k2 == "tier1" else "2_Personalized"
    group_names = test_group_weights.get(k1, {}).get(k2, {}).get("names", ["n/a"])
    weights = test_group_weights.get(k1, {}).get(k2, {}).get("weights", [1.0])
    topk = test_group_weights.get(k1, {}).get(k2, {}).get("topk", [5])
    assert len(group_names) == len(weights), "incomparable length groups and weights"
    # groups = [
    #     random.choices(population=group_name, weights=weight)[0]
    #     for group_name, weight in zip(group_names, weights)
    # ]
    test_group = random.choices(population=group_names, weights=weights, k=1)[0]
    topk_weekly = topk[group_names.index(test_group)]
    default_weekly = default_offers.get(k2, {}).get(test_group, [])
    return price_tier, test_group, topk_weekly, default_weekly



import os
import logging
from datetime import datetime
import pandas as pd
# import torch
import boto3
# from torch.utils.data import Dataset


# def get_device(device="auto") -> torch.device:
#     """
#     Retrieve PyTorch device.
#     It checks that the requested device is available first.
#     For now, it supports only cpu and cuda.
#     By default, it tries to use the gpu.
#     :param device: One for 'auto', 'cuda', 'cpu'
#     :return:
#     """
#     # Cuda by default
#     if device == "auto":
#         device = "cuda"
#     # Force conversion to torch.device
#     device = torch.device(device)
#
#     # Cuda not available
#     if device.type == torch.device("cuda").type and not torch.cuda.is_available():
#         return torch.device("cpu")
#
#     return device


# class OfferDatasetS3(Dataset):
#     # TODO: add 0 as missing store id into store_lookup in databricks
#     def __init__(self, s3_bucket, s3_data, s3_offer_lookup, s3_platform_lookup, s3_dma_lookup,
#                  s3_device_carrier_lookup, s3_device_model_lookup, parts_num=999):
#         self.logger = logging.getLogger("order_dataset")
#         self.logger.info("Load data and lookup tables directly from s3")
#         if parts_num == 0:
#             self.data = None
#         else:
#             self.data = self.load_s3_data(s3_bucket, s3_data, parts_num)
#         self.offer_lookup = self.load_s3_lookup(s3_bucket, s3_offer_lookup, file_type="json")
#         self.platform_lookup = self.load_s3_lookup(s3_bucket, s3_platform_lookup)
#         self.dma_lookup = self.load_s3_lookup(s3_bucket, s3_dma_lookup)
#         self.device_carrier_lookup = self.load_s3_lookup(s3_bucket, s3_device_carrier_lookup)
#         self.device_model_lookup = self.load_s3_lookup(s3_bucket, s3_device_model_lookup)
#
#     def __len__(self):
#         return len(self.data)
#
#     def __getitem__(self, idx):
#         context_col_names = ["platform_idx", "device_carrier_idx", "device_model_idx"]  # TODO: dma_idx
#         if torch.is_tensor(idx):
#             idx = idx.tolist()
#         order_context = [torch.tensor([self.data.iloc[idx][k]]) for k in context_col_names]
#         order_seq = torch.tensor(self.data.iloc[idx]["offer_idx"])
#         order_target = self.data.iloc[idx]["target_offer_idx"]
#         return order_context, order_seq, order_target
#
#     def load_s3_data(self, s3_bucket, s3_data, parts_num):
#         keys = []
#         dfs = []
#         s3 = boto3.client('s3')
#         list_obj = s3.list_objects_v2(Bucket=s3_bucket, Prefix=s3_data)
#         if list_obj["KeyCount"] > 1000:
#             self.logger.error("Found more than 1000 keys and only read partial data")
#         for content in list_obj["Contents"]:
#             if content["Key"].endswith(".json"):
#                 keys.append(content["Key"])
#         self.logger.info("Load %d json files from %s" % (min(parts_num, len(keys)), os.path.dirname(keys[0])))
#         for i, key in enumerate(keys):
#             if i < parts_num:
#                 obj = s3.get_object(Bucket=s3_bucket, Key=key)
#                 dfs.append(pd.read_json(obj["Body"], orient="columns", lines=True))
#         return pd.concat(dfs)
#
#     def load_s3_lookup(self, s3_bucket, s3_lookup, file_type="csv"):
#         keys = []
#         s3 = boto3.client('s3')
#         list_obj = s3.list_objects_v2(Bucket=s3_bucket, Prefix=s3_lookup)
#         if list_obj["KeyCount"] > 1000:
#             self.logger.error("Found more than 1000 keys and only read partial data")
#         for content in list_obj["Contents"]:
#             if content["Key"].endswith(file_type):
#                 keys.append(content["Key"])
#         if len(keys) > 1:
#             self.logger.error("Found more than 1 lookup %s" % file_type)
#         obj = s3.get_object(Bucket=s3_bucket, Key=keys[0])
#         self.logger.info("Load lookup table from %s" % keys[0])
#         if file_type == "csv":
#             df = pd.read_csv(obj["Body"])
#         elif file_type == "json":
#             df = pd.read_json(obj["Body"])
#         else:
#             raise ValueError("file_type can only be \"csv\" or \"json\"")
#         return df


def generate_lookup_dict(df, key_col, value_col):
    """Generate int2int lookup dict from given dataframe."""
    return df[[key_col, value_col]].set_index(key_col).to_dict()[value_col]


def load_s3_table(s3_bucket, s3_prefix, file_type="csv"):
    logger = logging.getLogger("load_s3")
    keys = []
    s3 = boto3.client('s3')
    list_obj = s3.list_objects_v2(Bucket=s3_bucket, Prefix=s3_prefix)
    if list_obj["KeyCount"] > 1000:
        logger.error("Found more than 1000 keys and only read partial data")
    for content in list_obj["Contents"]:
        if content["Key"].endswith(file_type):
            keys.append(content["Key"])
    if len(keys) > 1:
        logger.error("Found more than 1 lookup %s" % file_type)
    obj = s3.get_object(Bucket=s3_bucket, Key=keys[0])
    logger.info("Load s3 table from %s" % keys[0])
    if file_type == "csv":
        df = pd.read_csv(obj["Body"])
    elif file_type == "json":
        df = pd.read_json(obj["Body"], lines=True)
    else:
        raise ValueError("file_type can only be \"csv\" or \"json\"")
    return df


def item2name(item_code, item_lookup):
    item_code = int(item_code)
    if item_code == 0:
        return None
    else:
        item_names = item_lookup[item_lookup["item_code"] == item_code]["item_name_clean"]
        if len(item_names) != 1:
            print("WARNING - %d names were found for item %d" % (len(item_names), item_code))
        else:
            return item_names.iloc[0]




from torch.utils.data import Dataset, IterableDataset


class OrderDatasetLocalIterator(IterableDataset):
    """
    TODO: https://pytorch.org/docs/stable/data.html#torch.utils.data.IterableDataset
    """
    def __init__(self):
        pass

    def __len__(self):
        pass

    def __getitem__(self, idx):
        pass
